##----------------------------------------------------------------------------##
##=Purpose: Exercise 3.2 of Beyond Classic Designs & Analysis for Health      ##
##          Data in Singapore (27 June to 8 July 2022)                        ##
##
##=Data required: 
##  - full_cohort.dta
##
##=R packages/libraries required:
##  - dplyr: 
##    * for processing data
##  - Epi:
##    * ccwc: generate a nested case-control study
##    Note that this package contains clogistic that does conditional logistic
##  - haven:
##    * read_dta: To read in recent versions of STATA dta files.
##  - Matching:
##    * Match: to perform matched case-control 
##  - survival:
##    * clogit: conditional logistic
##    * coxph: Cox proportional hazard
##
##=Created by:
##  - Francesca Graziano 
##  - Zilu Feng
##  - Zihan 
##  - Chuen Seng Tan
##  - Marie Reilly
##
##=Changes (if any):
##  -DDMMYYYY: Describe
##
##----------------------------------------------------------------------------##

##-Set the working directory to the data folder where the datasets are saved:
setwd("D:/nbox/Education/Marie_short_course/Singapore course 2022/20220602/exercises/data")

##-R libraries to install:
#install.packages(c("dplyr","Epi","haven","Matching","survival"))
##-Load the R libraries
library(dplyr)
library(Epi)
library(haven)
library(Matching)
library(survival)

# Alternative approach with self-written functions:
# For simplicity, we define the following functions to fit the naive logistic 
# regression, weighted logistic regression for second-stage data, and logistic 
# regression for two-stage data using the survey package.
# 
#' Fit a Cox regression and output a table of the estimated HR, the 95% CI and
#' the p-value
#' @param ... Input to the function `coxph`, i.e., specify input to this
#'   function in the same way as to function `coxph`.
#' @return Returns the fitted model and prints the table of estimated HR, 95% CI
#'   and p-value. The coefficient matrix with HR is saved as an element of the 
#'   fitted model with the name `coef_with_or`. 
fit_cox <- function(...) {
  model <- coxph(...)
  tmp <- as.data.frame(summary(model)$coef)
  tmp$lower.CI <- exp(tmp$coef - 1.96 * tmp$`se(coef)`)
  tmp$upper.CI <- exp(tmp$coef + 1.96 * tmp$`se(coef)`)
  names(tmp)[c(2, 3)] <- c("HR", "SE")
  model$coef_with_or <- tmp
  print(knitr::kable(tmp[, c("HR", "SE", "lower.CI", "upper.CI", "Pr(>|z|)")], 
                     format = "pandoc", digits = 3, 
                     col.names = c("HR", "SE", "Lower 95% CI",
                                   "Upper 95% CI", "p-value"), 
                     caption = "Cox regression using coxph"))
  model
}

#' Fit a logistic regression (unweighted) and output a table of the estimated
#' OR, the 95% CI and the p-value
#' @param ... Input to the function `glm`, i.e., specify input to this function
#'   in the same way as to function `glm`.
#' @return Returns the fitted model and prints the table of estimated OR, 95% CI
#'   and p-value. The coefficient matrix with OR is saved as an element of the 
#'   fitted model with the name `coef_with_or`. 
fit_logistic <- function(...) {
  model <- glm(...)
  tmp <- as.data.frame(summary(model)$coef)
  tmp$OR <- exp(tmp[,1])
  tmp$lower.CI <- exp(tmp[,1] - 1.96 * tmp[,2])
  tmp$upper.CI <- exp(tmp[,1] + 1.96 * tmp[,2])
  names(tmp)[c(2, 4)] <- c('SE', 'p-value')
  model$coef_with_or <- tmp
  print(knitr::kable(tmp[,c('OR','SE','lower.CI','upper.CI','p-value')],
                     digits = 3, format = "pandoc", 
                     col.names = c("OR", "SE", "Lower 95% CI",
                                   "Upper 95% CI", "p-value"), 
                     caption = "Logistic regression using glm"))
  model
}
#' Fit a conditional logistic regression using `clogit` and output a table of
#' the estimated OR, the 95% CI and the p-value
#' @param ... Input to the function `clogit`, i.e., specify input to this
#'   function in the same way as to function `clogit`.
#' @return Returns the fitted model and prints the table of estimated OR, 95% CI
#'   and p-value. The coefficient matrix with OR is saved as an element of the
#'   fitted model with the name `coef_with_or`.
fit_clogit <- function(...) {
  model <- clogit(...)
  tmp <- as.data.frame(summary(model)$coef)
  tmp$lower.CI <- exp(tmp$coef - 1.96 * tmp$`se(coef)`)
  tmp$upper.CI <- exp(tmp$coef + 1.96 * tmp$`se(coef)`)
  names(tmp)[c(2, 3)] <- c("OR", "SE")
  model$coef_with_or <- tmp
  print(knitr::kable(tmp[, c("OR", "SE", "lower.CI", "upper.CI", "Pr(>|z|)")], 
                     format = "pandoc", digits = 3, 
                     col.names = c("HR", "SE", "Lower 95% CI",
                                   "Upper 95% CI", "p-value"), 
                     caption = "Conditional logistic regression using clogit"))
  model
}


################################################################################
##                               Question 1      
##
##-1. This exercise makes use the dataset full_cohort.dta, which is a simulated 
##    data set  of 50,000 cohort members, where the outcome of interest is time 
##    from enrolment to coronary heart disease with event indicator. The 
##    variables recorded for this cohort include: 
##
##    * t time from enrolment into the cohort to coronary heart disease (in years)
##    * event (event indicator 1=coronary heart disease, 0=censored)
##    * Age (Age, in years)
##    * Gender (1=Male, 0=Female)
##    * Chol (cholesterol in mg/dL)
##    * HDL (high-density lipoprotein in mg/dL)
##    * SBP (systolic blood pressure in mmHg)
##    * Treat (antihypertensive treatment status 1=Yes, 0=No) 
##    * Smoke (smoking status 1=Yes, 0=No)
################################################################################

################################################################################
##                               Question 1 (i)     
##
##-(i)	Create a categorical variable for age with the following intervals: 
##      <=49, 50-59, 60-69, 70-79, ???80
################################################################################

#import dataset 
full_cohort <- haven::read_dta("full_cohort.dta")

#create the categorical variable
full_cohort$Age_cat<-cut(full_cohort$Age,breaks=c(0,49,59,69,79,100))
table(full_cohort$Age_cat)


################################################################################
##                               Question 1 (ii)     
##
##-(ii)	For each of the other continuous covariates (Chol, HDL, and SBP) 
##      center them at their mean values (i.e. subtract the mean from each value). 
################################################################################

#center the variables at their mean values
full_cohort <- full_cohort %>% 
  mutate(Chol = Chol - mean(Chol), 
         HDL = HDL - mean(HDL), 
         SBP = SBP - mean(SBP))


################################################################################
##                               Question 1 (iii)     
##
##-(iii) Run a Cox regression analysis of the whole cohort with these new 
##       variables created in part (i) and (ii) and the remaining binary 
##       variables (Gender: 1=Male, 0=Female, Treat: 1=Yes, 0=No, 
##       Smoke: 1=Yes, 0=No). 
################################################################################

#run cox with whole cohort
cox_whole <- coxph(Surv(t, event) ~ Chol + HDL + SBP + 
                     Gender + Treat + Smoke + Age_cat, 
                   data = full_cohort)
#key results
round(cbind(exp(cbind(HR=cox_whole$coefficients,
                      confint(cox_whole))),
            p.value=summary(cox_whole)$coefficients[,"Pr(>|z|)"]),3)

#Alternatively (after sourcing the self-written functions at the top of R script):
fit_cox(Surv(t, event) ~ Chol + HDL + SBP + 
          Gender + Treat + Smoke + Age_cat, 
        data = full_cohort)

################################################################################
##                               Question 1 (iv)     
##
##-(iv) Assume the investigators wanted to study coronary heart disease by year 
##      10 by sampling an exclusive case-control dataset and conducting a 
##      logistic regression. Draw an exclusive 1:1 case-control sample at year 
##      10 that is matched on sex and age category. Save a copy of this data set 
##      for part (v).
################################################################################
#generate the complete case-control data and binary outcome at year 10
full_binary <- full_cohort %>%
  dplyr::filter((t>10 & event==0) | (event==1)) %>%
  dplyr::mutate(binary=ifelse(t>10 & event==1,0,event))

#check that the extraction and modification of binary outcome is sensible
tapply(full_binary$t,paste(full_binary$event,full_binary$binary),summary)
table(full_binary$event,full_binary$binary)

#using Match() function from Matching() library do matching of cases
#Tr = "treatment, i.e. outcome"
#M = "number of controls to be matched per case
#X = "covariate to match using in matrix"
#ties = FALSE so that when there are more than one possibility for a control, it will select a random control
#exact = Logical indicators on which variable (base on column number) has exact matching.
#replace = FALSE so that once a record has been selected for control, it can't be selected again
set.seed(2307)
cc_match <- Matching::Match(Tr = full_binary$binary,
                            M = 1,
                            X = cbind(full_binary$Age_cat,full_binary$Gender),
                            ties = FALSE,
                            exact = c(T,T),
                            replace = FALSE)
summary(cc_match)
#Identify the cases and which case-control pair they are in (matchset)
ii<-match(cc_match$index.treated,1:nrow(full_binary))
all(cc_match$index.treated==ii)
cases<-cbind(matchset=1:length(ii),full_binary[ii,])

#Identify the controls and which case-control pair they are in (matchset)
jj<-match(cc_match$index.control,1:nrow(full_binary))
all(cc_match$index.control==jj)
controls<-cbind(matchset=1:length(jj),full_binary[jj,])

#Have the cases and controls identified into a new dataset 
matched.fin <- rbind(cases,controls)

save(matched.fin, file="ex3_2.Rdata")


################################################################################
##                               Question 1 (v)     
##
##-(v) Run a conditional logistic regression analysis of the data from (iv) and 
##     compare the results to the full cohort in part (iii).
################################################################################

#run conditional logistic regression with case-control data
clogit_cc_matched <- clogit(binary ~ Chol + HDL + SBP + 
                              Treat + Smoke + strata(matchset), 
                            data = matched.fin)
#key results
round(cbind(exp(cbind(OR=clogit_cc_matched$coefficients,
                      confint(clogit_cc_matched))),
            p.value=summary(clogit_cc_matched)$coefficients[,"Pr(>|z|)"]),3)

#Alternatively (after sourcing the self-written functions at the top of R script):
fit_clogit(binary ~ Chol + HDL + SBP + 
             Treat + Smoke + strata(matchset), 
           data = matched.fin)


################################################################################
##                               Question 2 (i)     
##
##-(i) Run Cox regression analyses of the whole cohort to obtain the crude HR 
##     for Gender and the HR adjusted for age-category. 
################################################################################

#run cox regression to get crude HR for gender   
cox_whole2 <- coxph(Surv(t, event) ~  Gender , 
                      data = full_cohort)
#key results
round(cbind(exp(cbind(HR=cox_whole2$coefficients,
                      confint(cox_whole2))),
            p.value=summary(cox_whole2)$coefficients[,"Pr(>|z|)"]),3)

#run cox regression to getage-adjusted HR for age
cox_whole3 <- coxph(Surv(t, event) ~  Gender + Age_cat, 
                      data = full_cohort)
#key results
round(cbind(exp(cbind(HR=cox_whole3$coefficients,
                      confint(cox_whole3))),
            p.value=summary(cox_whole3)$coefficients[,"Pr(>|z|)"]),3)


#Alternatively (after sourcing the self-written functions at the top of R script):
#run cox regression to get crude HR for gender   
fit_cox(Surv(t, event) ~  Gender , 
        data = full_cohort)
#run cox regression to getage-adjusted HR for age
fit_cox(Surv(t, event) ~  Gender + Age_cat, 
        data = full_cohort)

################################################################################
##                               Question 2 (ii)     
##
##-(ii) Draw a 1:5 nested case-control sample (i.e. 5 controls for each case) 
##      using simple random sampling (use a seed!) and save a copy of the data 
##      before proceeding.  Run a conditional logistic regression of Gender 
##      using your nested case-control sample and compare the results with the 
##      crude HR for gender obtained from the whole cohort Cox regression.
################################################################################

#Draw a 1:5 nested case-control sample and save a copy of the data before proceeding
set.seed(2307)
ncc1 <- ccwc(exit = t, fail = event, controls = 5,
             include = list(id, Gender, t, Age, Chol, HDL, SBP, 
                            Treat, Smoke, Age_cat),
             data = full_cohort)
save(ncc1,file="ncc1.rdata")


#run a conditional logistic regression of Gender using nested case-control sample 
clogit_ncc1 <- clogit(Fail~Gender + strata(Set), 
                      data = ncc1)
#key results
round(cbind(exp(cbind(OR=clogit_ncc1$coefficients,
                      confint(clogit_ncc1))),
            p.value=summary(clogit_ncc1)$coefficients[,"Pr(>|z|)"]),3)

#Alternatively (after sourcing the self-written functions at the top of R script):
fit_clogit(Fail~Gender+strata(Set), 
       data = ncc1)


################################################################################
##                               Question 2 (iii)     
##
##-(iii) Draw a nested case-control sample that is matched on age category. 
##       Compare the OR for gender from a  conditional logistic regression 
##       analysis to the age-adjusted HR from the Cox regression of the whole 
##       cohort.
################################################################################

#draw a nested case- control sample that is matched on age category
ncc2 <- ccwc(exit = t, fail = event, controls = 5, match=Age_cat,
             include = list(id, Gender, t, Age, Age_cat),
             data = full_cohort)

#run conditional logistic regression analysis
clogit_ncc2 <- clogit(Fail~Gender + strata(Set), data = ncc2)
#key results
round(cbind(exp(cbind(OR=clogit_ncc2$coefficients,
                      confint(clogit_ncc2))),
            p.value=summary(clogit_ncc2)$coefficients[,"Pr(>|z|)"]),3)

#Alternatively (after sourcing the self-written functions at the top of R script):
fit_clogit(Fail~Gender+strata(Set), data = ncc2)

#See 2 (i) for age-adjusted HR from the Cox regression of the whole cohort

################################################################################
##                               Question 2 (iv)     
##
##-(iv) Use the nested case-control data from (ii) to run the full model in 
##      Question 1 (i.e., all the variables listed in part (iii), categorical 
##      age, mean centered HDL, Chol and SBP, and binary treatment, gender and 
##      smoking), and compare the results to the Cox regression of the whole 
##      cohort. 
################################################################################

#Use the nested case-control data from 2 (ii)
clogit_ncc3 <- clogit(Fail~Gender + Chol + HDL + SBP + 
                        Treat + Smoke + Age_cat + strata(Set), 
                      data = ncc1)
#key results
round(cbind(exp(cbind(OR=clogit_ncc3$coefficients,
                      confint(clogit_ncc3))),
            p.value=summary(clogit_ncc3)$coefficients[,"Pr(>|z|)"]),3)

#Alternatively (after sourcing the self-written functions at the top of R script):
fit_clogit(Fail~Gender + Chol + HDL + SBP + 
             Treat + Smoke + Age_cat+strata(Set), 
           data = ncc1)
